@@ -308,6 +308,7 @@ struct query
     struct idlist *skipped;	/* messages skipped on the mail server */
     struct idlist *oldsaved, *newsaved;
     char *lastid;		/* last Message-ID seen on this connection */
+    char *thisid;		/* Message-ID of current message */
 
     /* internal use -- per-message state */
     int mimemsg;		/* bitmask indicating MIME body-type */
@@ -505,7 +506,7 @@ int prc_filecheck(const char *, const flag);
 
 /* base64.c */
 void to64frombits(unsigned char *, const unsigned char *, int);
-int from64tobits(char *, const char *);
+int from64tobits(char *, const char *, int len);
 
 /* unmime.c */
 /* Bit-mask returned by MimeBodyType */
@@ -539,8 +540,8 @@ char *xstrdup(const char *);
 #endif
 #endif
 #define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n)))\
-       {report(stderr, _("alloca failed")); exit(PS_UNDEFINED);}
-#if FALSE
+       {report(stderr, GT_("alloca failed")); exit(PS_UNDEFINED);}
+#if 0
 /*
  * This is a hack to help xgettext which cannot find strings in
  * macro definitions like the one for xalloca above.
