@@ -181,7 +181,7 @@ static int do_imap_ntlm(int sock, struct query *ctl)
     if ((gen_recv(sock, msgbuf, sizeof msgbuf)))
 	return result;
   
-    len = from64tobits ((unsigned char*)&challenge, msgbuf);
+    len = from64tobits ((unsigned char*)&challenge, msgbuf, sizeof(msgbuf));
     
     if (outlevel >= O_DEBUG)
 	dumpSmbNtlmAuthChallenge(stdout, &challenge);
@@ -236,27 +236,32 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
     capabilities[0] = '\0';
     if ((ok = gen_transact(sock, "CAPABILITY")) == PS_SUCCESS)
     {
+	char	*cp;
+
+	/* capability checks are supposed to be caseblind */
+	for (cp = capabilities; *cp; cp++)
+	    *cp = toupper(*cp);
+
 	/* UW-IMAP server 10.173 notifies in all caps, but RFC2060 says we
 	   should expect a response in mixed-case */
-	if (strstr(capabilities, "IMAP4REV1") ||
-	    strstr(capabilities, "IMAP4rev1"))
+	if (strstr(capabilities, "IMAP4REV1"))
 	{
 	    imap_version = IMAP4rev1;
 	    if (outlevel >= O_DEBUG)
-		report(stdout, _("Protocol identified as IMAP4 rev 1\n"));
+		report(stdout, GT_("Protocol identified as IMAP4 rev 1\n"));
 	}
 	else
 	{
 	    imap_version = IMAP4;
 	    if (outlevel >= O_DEBUG)
-		report(stdout, _("Protocol identified as IMAP4 rev 0\n"));
+		report(stdout, GT_("Protocol identified as IMAP4 rev 0\n"));
 	}
     }
     else if (ok == PS_ERROR)
     {
 	imap_version = IMAP2;
 	if (outlevel >= O_DEBUG)
-	    report(stdout, _("Protocol identified as IMAP2 or IMAP2BIS\n"));
+	    report(stdout, GT_("Protocol identified as IMAP2 or IMAP2BIS\n"));
     }
     else
 	return(ok);
@@ -280,7 +285,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
     {
 	do_idle = TRUE;
 	if (outlevel >= O_VERBOSE)
-	    report(stdout, _("will idle after poll\n"));
+	    report(stdout, GT_("will idle after poll\n"));
     }
 
     /* 
@@ -366,10 +371,10 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
 	else
 	    return ok;
 #else
-    if (ctl->server.authenticate == A_NTLM)
+    if (ctl->server.authenticate == A_OTP)
     {
 	report(stderr, 
-	   _("Required OTP capability not compiled into fetchmail\n"));
+	   GT_("Required OTP capability not compiled into fetchmail\n"));
     }
 #endif /* OPIE_ENABLE */
 
@@ -391,7 +396,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
     if (ctl->server.authenticate == A_NTLM)
     {
 	report(stderr, 
-	   _("Required NTLM capability not compiled into fetchmail\n"));
+	   GT_("Required NTLM capability not compiled into fetchmail\n"));
     }
 #endif /* NTLM_ENABLE */
 
@@ -400,7 +405,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
     if ((imap_version >= IMAP4rev1) && (!strstr(capabilities, "LOGIN")))
     {
 	report(stderr, 
-	       _("Required LOGIN capability not supported by server\n"));
+	       GT_("Required LOGIN capability not supported by server\n"));
     }
 #endif /* __UNUSED__ */
 
@@ -488,7 +493,7 @@ static int imap_getrange(int sock,
 	    ok = imap_idle(sock);
 	if (ok || gen_transact(sock, "NOOP"))
 	{
-	    report(stderr, _("re-poll failed\n"));
+	    report(stderr, GT_("re-poll failed\n"));
 	    return(ok);
 	}
 	else if (count == -1)	/* no EXISTS response to NOOP/IDLE */
@@ -496,7 +501,7 @@ static int imap_getrange(int sock,
 	    count = 0;
 	}
 	if (outlevel >= O_DEBUG)
-	    report(stdout, _("%d messages waiting after re-poll\n"), count);
+	    report(stdout, GT_("%d messages waiting after re-poll\n"), count);
     }
     else
     {
@@ -505,11 +510,11 @@ static int imap_getrange(int sock,
 			  folder ? folder : "INBOX");
 	if (ok != 0)
 	{
-	    report(stderr, _("mailbox selection failed\n"));
+	    report(stderr, GT_("mailbox selection failed\n"));
 	    return(ok);
 	}
 	else if (outlevel >= O_DEBUG)
-	    report(stdout, _("%d messages waiting after first poll\n"), count);
+	    report(stdout, GT_("%d messages waiting after first poll\n"), count);
 
 	/* no messages?  then we may need to idle until we get some */
 	if (count == 0 && do_idle)
@@ -532,7 +537,7 @@ static int imap_getrange(int sock,
 	    ok = gen_recv(sock, buf, sizeof(buf));
 	    if (ok != 0)
 	    {
-		report(stderr, _("search for unseen messages failed\n"));
+		report(stderr, GT_("search for unseen messages failed\n"));
 		return(PS_PROTOCOL);
 	    }
 	    else if ((cp = strstr(buf, "* SEARCH")))
@@ -556,7 +561,7 @@ static int imap_getrange(int sock,
 
 			if (outlevel >= O_DEBUG)
 			    report(stdout, 
-				   _("%u is unseen\n"), 
+				   GT_("%u is unseen\n"), 
 				   unseen_messages[unseen]);
 		
 			unseen++;
@@ -620,14 +625,19 @@ static int imap_getsizes(int sock, int count, int *sizes)
 	gen_send(sock, "FETCH 1:%d RFC822.SIZE", count);
     for (;;)
     {
-	int num, size, ok;
+	unsigned int num, size;
+	int ok;
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
 	    return(ok);
 	else if (strstr(buf, "OK") || strstr(buf, "NO"))
 	    break;
-	else if (sscanf(buf, "* %d FETCH (RFC822.SIZE %d)", &num, &size) == 2)
-	    sizes[num - 1] = size;
+	else if (sscanf(buf, "* %u FETCH (RFC822.SIZE %u)", &num, &size) == 2) {
+	    if (num > 0 && num <= count)
+	        sizes[num - 1] = size;
+	    else
+		report(stderr, "Warning: ignoring bogus data for message sizes returned by the server.\n");
+	}
     }
 
     return(PS_SUCCESS);
@@ -656,6 +666,14 @@ static int imap_is_old(int sock, struct query *ctl, int number)
     return(seen);
 }
 
+static char *skip_token(char *ptr)
+{
+    while(isspace(*ptr)) ptr++;
+    while(!isspace(*ptr) && !iscntrl(*ptr)) ptr++;
+    while(isspace(*ptr)) ptr++;
+    return(ptr);
+}
+
 static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
 /* request headers of nth message */
 {
@@ -672,13 +690,22 @@ static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
     gen_send(sock, "FETCH %d RFC822.HEADER", number);
 
     /* looking for FETCH response */
-    do {
+    for (;;) 
+    {
 	int	ok;
+	char	*ptr;
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
 	    return(ok);
-    } while
-	(sscanf(buf+2, "%d FETCH (%*s {%d}", &num, lenp) != 2);
+ 	ptr = skip_token(buf);	/* either "* " or "AXXXX " */
+ 	if (sscanf(ptr, "%d FETCH (%*s {%d}", &num, lenp) == 2)
+  	    break;
+	/* try to recover from chronically fucked-up M$ Exchange servers */
+ 	else if (!strncmp(ptr, "NO", 2))
+ 	    return(PS_TRANSIENT);
+ 	else if (!strncmp(ptr, "BAD", 3))
+ 	    return(PS_TRANSIENT);
+    }
 
     if (num != number)
 	return(PS_ERROR);
