@@ -49,6 +49,8 @@ static int lmtp_responses;
 int smtp_open(struct query *ctl)
 /* try to open a socket to the appropriate SMTP server for this query */ 
 {
+    char *parsed_host = NULL;
+
     /* maybe it's time to close the socket in order to force delivery */
     if (NUM_NONZERO(ctl->batchlimit) && (ctl->smtp_socket != -1) && ++batchcount == ctl->batchlimit)
     {
@@ -95,7 +97,7 @@ int smtp_open(struct query *ctl)
 	set_timeout(ctl->server.timeout);
 	for (idp = ctl->smtphunt; idp; idp = idp->next)
 	{
-	    char	*cp, *parsed_host;
+	    char	*cp;
 #ifdef INET6_ENABLE 
 	    char	*portnum = SMTP_PORT;
 #else
@@ -120,12 +122,12 @@ int smtp_open(struct query *ctl)
 	    }
 
 	    if (ctl->smtphost[0]=='/'){
-		if((ctl->smtp_socket = UnixOpen(ctl->smtphost))==-1)
+		if ((ctl->smtp_socket = UnixOpen(ctl->smtphost))==-1)
 		    continue;
 	    } else
-	    if ((ctl->smtp_socket = SockOpen(parsed_host,portnum,NULL,
+		if ((ctl->smtp_socket = SockOpen(parsed_host,portnum,NULL,
 					     ctl->server.plugout)) == -1)
-		continue;
+		    continue;
 
 	    /* return immediately for ODMR */
 	    if (ctl->server.protocol == P_ODMR)
@@ -169,91 +171,26 @@ int smtp_open(struct query *ctl)
      * or MX but not a CNAME.  Some listeners (like exim)
      * enforce this.  Now that we have the actual hostname,
      * compute what we should canonicalize with.
+     * 
+     * make sure we do not forget to drop the /port if
+     * using LMTP (hmh)
      */
-    ctl->destaddr = ctl->smtpaddress ? ctl->smtpaddress : ( ctl->smtphost && ctl->smtphost[0] != '/' ? ctl->smtphost : "localhost");
+    if (ctl->listener == LMTP_MODE && !ctl->smtpaddress) 
+    {
+	if (parsed_host && parsed_host[0] != 0)
+		ctl->destaddr = xstrdup(parsed_host);
+	else 
+		ctl->destaddr = (ctl->smtphost && ctl->smtphost[0] != '/') ? ctl->smtphost : "localhost";
+    } 
+    else 
+	ctl->destaddr = ctl->smtpaddress ? ctl->smtpaddress : ( ctl->smtphost && ctl->smtphost[0] != '/' ? ctl->smtphost : "localhost");
 
     if (outlevel >= O_DEBUG && ctl->smtp_socket != -1)
-	report(stdout, _("forwarding to %s\n"), ctl->smtphost);
+	report(stdout, GT_("forwarding to %s\n"), ctl->smtphost);
 
     return(ctl->smtp_socket);
 }
 
-/* these are shared by open_sink and stuffline */
-static FILE *sinkfp;
-
-int stuffline(struct query *ctl, char *buf)
-/* ship a line to the given control block's output sink (SMTP server or MDA) */
-{
-    int	n, oldphase;
-    char *last;
-
-    /* The line may contain NUL characters. Find the last char to use
-     * -- the real line termination is the sequence "\n\0".
-     */
-    last = buf;
-    while ((last += strlen(last)) && (last[-1] != '\n'))
-        last++;
-
-    /* fix message lines that have only \n termination (for qmail) */
-    if (ctl->forcecr)
-    {
-        if (last - 1 == buf || last[-2] != '\r')
-	{
-	    last[-1] = '\r';
-	    *last++  = '\n';
-	    *last    = '\0';
-	}
-    }
-
-    oldphase = phase;
-    phase = FORWARDING_WAIT;
-
-    /*
-     * SMTP byte-stuffing.  We only do this if the protocol does *not*
-     * use .<CR><LF> as EOM.  If it does, the server will already have
-     * decorated any . lines it sends back up.
-     */
-    if (*buf == '.')
-    {
-	if (ctl->server.base_protocol->delimited)	/* server has already byte-stuffed */
-	{
-	    if (ctl->mda)
-		++buf;
-	    else
-		/* writing to SMTP, leave the byte-stuffing in place */;
-	}
-        else /* if (!protocol->delimited)	-- not byte-stuffed already */
-	{
-	    if (!ctl->mda)
-		SockWrite(ctl->smtp_socket, buf, 1);	/* byte-stuff it */
-	    else
-		/* leave it alone */;
-	}
-    }
-
-    /* we may need to strip carriage returns */
-    if (ctl->stripcr)
-    {
-	char	*sp, *tp;
-
-	for (sp = tp = buf; sp < last; sp++)
-	    if (*sp != '\r')
-		*tp++ =  *sp;
-	*tp = '\0';
-        last = tp;
-    }
-
-    n = 0;
-    if (ctl->mda || ctl->bsmtp)
-	n = fwrite(buf, 1, last - buf, sinkfp);
-    else if (ctl->smtp_socket != -1)
-	n = SockWrite(ctl->smtp_socket, buf, last - buf);
-
-    phase = oldphase;
-
-    return(n);
-}
-
 static void sanitize(char *s)
 /* replace unsafe shellchars by an _ */
 {
@@ -303,10 +240,10 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 	    (int)getpid(), (int)getppid(), time((time_t *)NULL));
 
     if (outlevel >= O_VERBOSE)
-	report(stdout, _("SMTP: (bounce-message body)\n"));
+	report(stdout, GT_("SMTP: (bounce-message body)\n"));
     else
 	/* this will usually go to sylog... */
-	report(stderr, _("mail from %s bounced to %s\n"),
+	report(stderr, GT_("mail from %s bounced to %s\n"),
 	       daemon_name, bounce_to);
 
     /* bouncemail headers */
@@ -408,7 +345,7 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
      */
     SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
     if (outlevel >= O_DEBUG)
-	report(stdout, _("Saved error is still %d\n"), smtperr);
+	report(stdout, GT_("Saved error is still %d\n"), smtperr);
 #endif /* __UNUSED */
 
     /*
@@ -453,7 +390,7 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
      * an error when the return code is less specific.
      */
     if (smtperr >= 400)
-	report(stderr, _("%cMTP error: %s\n"), 
+	report(stderr, GT_("%cMTP error: %s\n"), 
 	      ctl->listener,
 	      responses[0]);
 
@@ -465,10 +402,11 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
 	 * ESMTP server.  Don't try to ship the message, 
 	 * and allow it to be deleted.
 	 */
-	send_bouncemail(ctl, msg, XMIT_ACCEPT,
+	if (run.bouncemail)
+	    send_bouncemail(ctl, msg, XMIT_ACCEPT,
 			"This message was too large (SMTP error 552).\r\n", 
 			1, responses);
-	return(run.bouncemail ? PS_REFUSED : PS_TRANSIENT);
+	return(PS_REFUSED);
   
     case 553: /* invalid sending domain */
 	/*
@@ -478,9 +416,12 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
 	 * (b) we wouldn't want spammers to get confirmation that
 	 * this address is live, anyway.
 	 */
-	send_bouncemail(ctl, msg, XMIT_ACCEPT,
+#ifdef __DONT_FEED_THE_SPAMMERS__
+	if (run.bouncemail)
+	    send_bouncemail(ctl, msg, XMIT_ACCEPT,
 			"Invalid address in MAIL FROM (SMTP error 553).\r\n", 
 			1, responses);
+#endif /* __DONT_FEED_THE_SPAMMERS__ */
 	return(PS_REFUSED);
 
     default:
@@ -511,246 +452,543 @@ static int handle_smtp_report(struct query *ctl, struct msgblk *msg)
     }
 }
 
+/* these are shared by open_sink and stuffline */
+static FILE *sinkfp;
+
+int stuffline(struct query *ctl, char *buf)
+/* ship a line to the given control block's output sink (SMTP server or MDA) */
+{
+    int	n, oldphase;
+    char *last;
+
+    /* The line may contain NUL characters. Find the last char to use
+     * -- the real line termination is the sequence "\n\0".
+     */
+    last = buf;
+    while ((last += strlen(last)) && (last[-1] != '\n'))
+        last++;
+
+    /* fix message lines that have only \n termination (for qmail) */
+    if (ctl->forcecr)
+    {
+        if (last - 1 == buf || last[-2] != '\r')
+	{
+	    last[-1] = '\r';
+	    *last++  = '\n';
+	    *last    = '\0';
+	}
+    }
+
+    oldphase = phase;
+    phase = FORWARDING_WAIT;
+
+    /*
+     * SMTP byte-stuffing.  We only do this if the protocol does *not*
+     * use .<CR><LF> as EOM.  If it does, the server will already have
+     * decorated any . lines it sends back up.
+     */
+    if (*buf == '.')
+    {
+	if (ctl->server.base_protocol->delimited)	/* server has already byte-stuffed */
+	{
+	    if (ctl->mda)
+		++buf;
+	    else
+		/* writing to SMTP, leave the byte-stuffing in place */;
+	}
+        else /* if (!protocol->delimited)	-- not byte-stuffed already */
+	{
+	    if (!ctl->mda)
+		SockWrite(ctl->smtp_socket, buf, 1);	/* byte-stuff it */
+	    else
+		/* leave it alone */;
+	}
+    }
+
+    /* we may need to strip carriage returns */
+    if (ctl->stripcr)
+    {
+	char	*sp, *tp;
+
+	for (sp = tp = buf; sp < last; sp++)
+	    if (*sp != '\r')
+		*tp++ =  *sp;
+	*tp = '\0';
+        last = tp;
+    }
+
+    n = 0;
+    if (ctl->mda || ctl->bsmtp)
+	n = fwrite(buf, 1, last - buf, sinkfp);
+    else if (ctl->smtp_socket != -1)
+	n = SockWrite(ctl->smtp_socket, buf, last - buf);
+
+    phase = oldphase;
+
+    return(n);
+}
+
+static int open_bsmtp_sink(struct query *ctl, struct msgblk *msg,
+	      int *good_addresses, int *bad_addresses)
+/* open a BSMTP stream */
+{
+    struct	idlist *idp;
+
+    if (strcmp(ctl->bsmtp, "-") == 0)
+	sinkfp = stdout;
+    else
+	sinkfp = fopen(ctl->bsmtp, "a");
+
+    /* see the ap computation under the SMTP branch */
+    fprintf(sinkfp, 
+	    "MAIL FROM: %s", (msg->return_path[0]) ? msg->return_path : user);
+
+    if (ctl->pass8bits || (ctl->mimemsg & MSG_IS_8BIT))
+	fputs(" BODY=8BITMIME", sinkfp);
+    else if (ctl->mimemsg & MSG_IS_7BIT)
+	fputs(" BODY=7BIT", sinkfp);
+
+    /* exim's BSMTP processor does not handle SIZE */
+    /* fprintf(sinkfp, " SIZE=%d", msg->reallen); */
+
+    fprintf(sinkfp, "\r\n");
+
+    /*
+     * RFC 1123 requires that the domain name part of the
+     * RCPT TO address be "canonicalized", that is a FQDN
+     * or MX but not a CNAME.  Some listeners (like exim)
+     * enforce this.  Now that we have the actual hostname,
+     * compute what we should canonicalize with.
+     */
+    ctl->destaddr = ctl->smtpaddress ? ctl->smtpaddress : "localhost";
+
+    *bad_addresses = 0;
+    for (idp = msg->recipients; idp; idp = idp->next)
+	if (idp->val.status.mark == XMIT_ACCEPT)
+	{
+	    if (ctl->smtpname)
+		fprintf(sinkfp, "RCPT TO: %s\r\n", ctl->smtpname);
+	    else if (strchr(idp->id, '@'))
+		fprintf(sinkfp,
+			"RCPT TO: %s\r\n", idp->id);
+	    else
+		fprintf(sinkfp,
+			"RCPT TO: %s@%s\r\n", idp->id, ctl->destaddr);
+	    *good_addresses = 0;
+	}
+
+    fputs("DATA\r\n", sinkfp);
+
+    if (ferror(sinkfp))
+    {
+	report(stderr, GT_("BSMTP file open or preamble write failed\n"));
+	return(PS_BSMTP);
+    }
+
+    return(PS_SUCCESS);
+}
+
+/* this is experimental and will be removed if double bounces are reported */
+#define EXPLICIT_BOUNCE_ON_BAD_ADDRESS
+
+static int open_smtp_sink(struct query *ctl, struct msgblk *msg,
+	      int *good_addresses, int *bad_addresses)
+/* open an SMTP stream */
+{
+    const char	*ap;
+    struct	idlist *idp;
+    char		options[MSGBUFSIZE]; 
+    char		addr[HOSTLEN+USERNAMELEN+1];
+#ifdef EXPLICIT_BOUNCE_ON_BAD_ADDRESS
+    char		**from_responses;
+#endif /* EXPLICIT_BOUNCE_ON_BAD_ADDRESS */
+    int		total_addresses;
+
+    /*
+     * Compute ESMTP options.
+     */
+    options[0] = '\0';
+    if (ctl->server.esmtp_options & ESMTP_8BITMIME) {
+	 if (ctl->pass8bits || (ctl->mimemsg & MSG_IS_8BIT))
+	    strcpy(options, " BODY=8BITMIME");
+	 else if (ctl->mimemsg & MSG_IS_7BIT)
+	    strcpy(options, " BODY=7BIT");
+    }
+
+    if ((ctl->server.esmtp_options & ESMTP_SIZE) && msg->reallen > 0)
+	sprintf(options + strlen(options), " SIZE=%d", msg->reallen);
+
+    /*
+     * Try to get the SMTP listener to take the Return-Path
+     * address as MAIL FROM.  If it won't, fall back on the
+     * remotename and mailserver host.  This won't affect replies,
+     * which use the header From address anyway; the MAIL FROM
+     * address is a place for the SMTP listener to send
+     * bouncemail.  The point is to guarantee a FQDN in the MAIL
+     * FROM line -- some SMTP listeners, like smail, become
+     * unhappy otherwise.
+     *
+     * RFC 1123 requires that the domain name part of the
+     * MAIL FROM address be "canonicalized", that is a
+     * FQDN or MX but not a CNAME.  We'll assume the Return-Path
+     * header is already in this form here (it certainly
+     * is if rewrite is on).  RFC 1123 is silent on whether
+     * a nonexistent hostname part is considered canonical.
+     *
+     * This is a potential problem if the MTAs further upstream
+     * didn't pass canonicalized From/Return-Path lines, *and* the
+     * local SMTP listener insists on them. 
+     *
+     * Handle the case where an upstream MTA is setting a return
+     * path equal to "@".  Ghod knows why anyone does this, but 
+     * it's been reported to happen in mail from Amazon.com and
+     * Motorola.
+     */
+    if (!msg->return_path[0] || (0 == strcmp(msg->return_path, "@")))
+    {
+#ifdef HAVE_SNPRINTF
+	snprintf(addr, sizeof(addr),
+#else
+	sprintf(addr,
+#endif /* HAVE_SNPRINTF */
+	      "%s@%s", ctl->remotename, ctl->server.truename);
+	ap = addr;
+    }
+    else if (strchr(msg->return_path,'@') || strchr(msg->return_path,'!'))
+	ap = msg->return_path;
+    else		/* in case Return-Path existed but was local */
+    {
+#ifdef HAVE_SNPRINTF
+	snprintf(addr, sizeof(addr),
+#else
+	sprintf(addr,
+#endif /* HAVE_SNPRINTF */
+		"%s@%s", msg->return_path, ctl->server.truename);
+	ap = addr;
+    }
+
+    if (SMTP_from(ctl->smtp_socket, ap, options) != SM_OK)
+    {
+	int err = handle_smtp_report(ctl, msg);
+
+	SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
+	return(err);
+    }
+
+    /*
+     * Now list the recipient addressees
+     */
+    total_addresses = 0;
+    for (idp = msg->recipients; idp; idp = idp->next)
+	total_addresses++;
+#ifdef EXPLICIT_BOUNCE_ON_BAD_ADDRESS
+    xalloca(from_responses, char **, sizeof(char *) * total_addresses);
+#endif /* EXPLICIT_BOUNCE_ON_BAD_ADDRESS */
+    for (idp = msg->recipients; idp; idp = idp->next)
+	if (idp->val.status.mark == XMIT_ACCEPT)
+	{
+	    if (strchr(idp->id, '@'))
+		strcpy(addr, idp->id);
+	    else {
+		if (ctl->smtpname) {
+#ifdef HAVE_SNPRINTF
+		    snprintf(addr, sizeof(addr)-1, "%s", ctl->smtpname);
+#else
+		    sprintf(addr, "%s", ctl->smtpname);
+#endif /* HAVE_SNPRINTF */
+
+		} else {
+#ifdef HAVE_SNPRINTF
+		  snprintf(addr, sizeof(addr)-1, "%s@%s", idp->id, ctl->destaddr);
+#else
+		  sprintf(addr, "%s@%s", idp->id, ctl->destaddr);
+#endif /* HAVE_SNPRINTF */
+		}
+	    }
+	    if (SMTP_rcpt(ctl->smtp_socket, addr) == SM_OK)
+		(*good_addresses)++;
+	    else
+	    {
+#ifdef EXPLICIT_BOUNCE_ON_BAD_ADDRESS
+ 		    char errbuf[POPBUFSIZE];
+#endif /* EXPLICIT_BOUNCE_ON_BAD_ADDRESS */
+		handle_smtp_report(ctl, msg);
+
+#ifdef EXPLICIT_BOUNCE_ON_BAD_ADDRESS
+#ifdef HAVE_SNPRINTF
+		snprintf(errbuf, sizeof(errbuf), "%s: %s",
+				idp->id, smtp_response);
+#else
+		strncpy(errbuf, idp->id, sizeof(errbuf));
+		strcat(errbuf, ": ");
+		strcat(errbuf, smtp_response);
+#endif /* HAVE_SNPRINTF */
+
+		xalloca(from_responses[*bad_addresses], 
+			char *, 
+			strlen(errbuf)+1);
+		strcpy(from_responses[*bad_addresses], errbuf);
+#endif /* EXPLICIT_BOUNCE_ON_BAD_ADDRESS */
+
+		(*bad_addresses)++;
+		idp->val.status.mark = XMIT_RCPTBAD;
+		if (outlevel >= O_VERBOSE)
+		    report(stderr, 
+			  GT_("%cMTP listener doesn't like recipient address `%s'\n"),
+			  ctl->listener, addr);
+	    }
+	}
+
+#ifdef EXPLICIT_BOUNCE_ON_BAD_ADDRESS
+    /*
+     * This should not be necessary, because the SMTP listener itself
+     * should genrate a bounce for the bad address.
+     */
+    if (*bad_addresses)
+	send_bouncemail(ctl, msg, XMIT_RCPTBAD,
+			"Some addresses were rejected by the MDA fetchmail forwards to.\r\n",
+			*bad_addresses, from_responses);
+#endif /* EXPLICIT_BOUNCE_ON_BAD_ADDRESS */
+
+    /*
+     * It's tempting to do local notification only if bouncemail was
+     * insufficient -- that is, to add && total_addresses > *bad_addresses
+     * to the test here.  The problem with this theory is that it would
+     * make initial diagnosis of a broken multidrop configuration very
+     * hard -- most single-recipient messages would just invisibly bounce.
+     */
+    if (!(*good_addresses)) 
+    {
+	if (!run.postmaster[0])
+	{
+	    if (outlevel >= O_VERBOSE)
+		report(stderr, GT_("no address matches; no postmaster set.\n"));
+	    SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
+	    return(PS_REFUSED);
+	}
+	if (strchr(run.postmaster, '@'))
+	    strncpy(addr, run.postmaster, sizeof(addr));
+	else
+	{
+#ifdef HAVE_SNPRINTF
+	    snprintf(addr, sizeof(addr)-1, "%s@%s", run.postmaster, ctl->destaddr);
+#else
+	    sprintf(addr, "%s@%s", run.postmaster, ctl->destaddr);
+#endif /* HAVE_SNPRINTF */
+	}
+
+	if (SMTP_rcpt(ctl->smtp_socket, addr) != SM_OK)
+	{
+	    report(stderr, GT_("can't even send to %s!\n"), run.postmaster);
+	    SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
+	    return(PS_REFUSED);
+	}
+
+	if (outlevel >= O_VERBOSE)
+	    report(stderr, GT_("no address matches; forwarding to %s.\n"), run.postmaster);
+    }
+
+    /* 
+     * Tell the listener we're ready to send data.
+     * Some listeners (like zmailer) may return antispam errors here.
+     */
+    if (SMTP_data(ctl->smtp_socket) != SM_OK)
+    {
+	SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
+	return(handle_smtp_report(ctl, msg));
+    }
+
+    /*
+     * We need to stash this away in order to know how many
+     * response lines to expect after the LMTP end-of-message.
+     */
+    lmtp_responses = *good_addresses;
+
+    return(PS_SUCCESS);
+}
+
+static int open_mda_sink(struct query *ctl, struct msgblk *msg,
+	      int *good_addresses, int *bad_addresses)
+/* open a stream to a local MDA */
+{
+#ifdef HAVE_SIGACTION
+    struct      sigaction sa_new;
+#endif /* HAVE_SIGACTION */
+    struct	idlist *idp;
+    int	length = 0, fromlen = 0, nameslen = 0;
+    char	*names = NULL, *before, *after, *from = NULL;
+
+    ctl->destaddr = "localhost";
+
+    for (idp = msg->recipients; idp; idp = idp->next)
+	if (idp->val.status.mark == XMIT_ACCEPT)
+	    (*good_addresses)++;
+
+    length = strlen(ctl->mda);
+    before = xstrdup(ctl->mda);
+
+    /* get user addresses for %T (or %s for backward compatibility) */
+    if (strstr(before, "%s") || strstr(before, "%T"))
+    {
+	/*
+	 * We go through this in order to be able to handle very
+	 * long lists of users and (re)implement %s.
+	 */
+	nameslen = 0;
+	for (idp = msg->recipients; idp; idp = idp->next)
+	    if ((idp->val.status.mark == XMIT_ACCEPT))
+		nameslen += (strlen(idp->id) + 1);	/* string + ' ' */
+	if ((*good_addresses == 0))
+	    nameslen = strlen(run.postmaster);
+
+	names = (char *)xmalloc(nameslen + 1);	/* account for '\0' */
+	if (*good_addresses == 0)
+	    strcpy(names, run.postmaster);
+	else
+	{
+	    names[0] = '\0';
+	    for (idp = msg->recipients; idp; idp = idp->next)
+		if (idp->val.status.mark == XMIT_ACCEPT)
+		{
+		    strcat(names, idp->id);
+		    strcat(names, " ");
+		}
+	    names[--nameslen] = '\0';	/* chop trailing space */
+	}
+
+	/* sanitize names in order to contain only harmless shell chars */
+	sanitize(names);
+    }
+
+    /* get From address for %F */
+    if (strstr(before, "%F"))
+    {
+	from = xstrdup(msg->return_path);
+
+	/* sanitize from in order to contain *only* harmless shell chars */
+	sanitize(from);
+
+	fromlen = strlen(from);
+    }
+
+    /* do we have to build an mda string? */
+    if (names || from) 
+    {		
+	char	*sp, *dp;
+
+	/* find length of resulting mda string */
+	sp = before;
+	while ((sp = strstr(sp, "%s"))) {
+	    length += nameslen - 2;	/* subtract %s */
+	    sp += 2;
+	}
+	sp = before;
+	while ((sp = strstr(sp, "%T"))) {
+	    length += nameslen - 2;	/* subtract %T */
+	    sp += 2;
+	}
+	sp = before;
+	while ((sp = strstr(sp, "%F"))) {
+	    length += fromlen - 2;	/* subtract %F */
+	    sp += 2;
+	}
+
+	after = xmalloc(length + 1);
+
+	/* copy mda source string to after, while expanding %[sTF] */
+	for (dp = after, sp = before; (*dp = *sp); dp++, sp++) {
+	    if (sp[0] != '%')	continue;
+
+	    /* need to expand? BTW, no here overflow, because in
+	    ** the worst case (end of string) sp[1] == '\0' */
+	    if (sp[1] == 's' || sp[1] == 'T') {
+		strcpy(dp, names);
+		dp += nameslen;
+		sp++;	/* position sp over [sT] */
+		dp--;	/* adjust dp */
+	    } else if (sp[1] == 'F') {
+		strcpy(dp, from);
+		dp += fromlen;
+		sp++;	/* position sp over F */
+		dp--;	/* adjust dp */
+	    }
+	}
+
+	if (names) {
+	    free(names);
+	    names = NULL;
+	}
+	if (from) {
+	    free(from);
+	    from = NULL;
+	}
+
+	free(before);
+
+	before = after;
+    }
+
+
+    if (outlevel >= O_DEBUG)
+	report(stdout, GT_("about to deliver with: %s\n"), before);
+
+#ifdef HAVE_SETEUID
+    /*
+     * Arrange to run with user's permissions if we're root.
+     * This will initialize the ownership of any files the
+     * MDA creates properly.  (The seteuid call is available
+     * under all BSDs and Linux)
+     */
+    seteuid(ctl->uid);
+#endif /* HAVE_SETEUID */
+
+    sinkfp = popen(before, "w");
+    free(before);
+    before = NULL;
+
+#ifdef HAVE_SETEUID
+    /* this will fail quietly if we didn't start as root */
+    seteuid(0);
+#endif /* HAVE_SETEUID */
+
+    if (!sinkfp)
+    {
+	report(stderr, GT_("MDA open failed\n"));
+	return(PS_IOERR);
+    }
+
+    /*
+     * We need to disable the normal SIGCHLD handling here because 
+     * sigchld_handler() would reap away the error status, returning
+     * error status instead of 0 for successful completion.
+     */
+#ifndef HAVE_SIGACTION
+    signal(SIGCHLD, SIG_DFL);
+#else
+    memset (&sa_new, 0, sizeof sa_new);
+    sigemptyset (&sa_new.sa_mask);
+    sa_new.sa_handler = SIG_DFL;
+    sigaction (SIGCHLD, &sa_new, NULL);
+#endif /* HAVE_SIGACTION */
+
+    return(PS_SUCCESS);
+}
+
 int open_sink(struct query *ctl, struct msgblk *msg,
 	      int *good_addresses, int *bad_addresses)
 /* set up sinkfp to be an input sink we can ship a message to */
 {
-    struct	idlist *idp;
-#ifdef HAVE_SIGACTION
-    struct      sigaction sa_new;
-#endif /* HAVE_SIGACTION */
-
     *bad_addresses = *good_addresses = 0;
 
     if (ctl->bsmtp)		/* dump to a BSMTP batch file */
-    {
-	if (strcmp(ctl->bsmtp, "-") == 0)
-	    sinkfp = stdout;
-	else
-	    sinkfp = fopen(ctl->bsmtp, "a");
-
-	/* see the ap computation under the SMTP branch */
-	fprintf(sinkfp, 
-		"MAIL FROM: %s", (msg->return_path[0]) ? msg->return_path : user);
-
-	if (ctl->pass8bits || (ctl->mimemsg & MSG_IS_8BIT))
-	    fputs(" BODY=8BITMIME", sinkfp);
-	else if (ctl->mimemsg & MSG_IS_7BIT)
-	    fputs(" BODY=7BIT", sinkfp);
-
-	/* exim's BSMTP processor does not handle SIZE */
-	/* fprintf(sinkfp, " SIZE=%d", msg->reallen); */
-
-	fprintf(sinkfp, "\r\n");
-
-	/*
-	 * RFC 1123 requires that the domain name part of the
-	 * RCPT TO address be "canonicalized", that is a FQDN
-	 * or MX but not a CNAME.  Some listeners (like exim)
-	 * enforce this.  Now that we have the actual hostname,
-	 * compute what we should canonicalize with.
-	 */
-	ctl->destaddr = ctl->smtpaddress ? ctl->smtpaddress : "localhost";
-
-	*bad_addresses = 0;
-	for (idp = msg->recipients; idp; idp = idp->next)
-	    if (idp->val.status.mark == XMIT_ACCEPT)
-	    {
-	        if (ctl->smtpname)
- 		    fprintf(sinkfp, "RCPT TO: %s\r\n", ctl->smtpname);
-		else if (strchr(idp->id, '@'))
-  		    fprintf(sinkfp,
-			    "RCPT TO: %s\r\n", idp->id);
-		else
-		    fprintf(sinkfp,
-			    "RCPT TO: %s@%s\r\n", idp->id, ctl->destaddr);
-		*good_addresses = 0;
-	    }
-
-	fputs("DATA\r\n", sinkfp);
-
-	if (ferror(sinkfp))
-	{
-	    report(stderr, _("BSMTP file open or preamble write failed\n"));
-	    return(PS_BSMTP);
-	}
-    }
-
+	return(open_bsmtp_sink(ctl, msg, good_addresses, bad_addresses));
     /* 
      * Try to forward to an SMTP or LMTP listener.  If the attempt to 
      * open a socket fails, fall through to attempt delivery via
      * local MDA.
      */
     else if (!ctl->mda && smtp_open(ctl) != -1)
-    {
-	const char	*ap;
-	char		options[MSGBUFSIZE]; 
-	char		addr[HOSTLEN+USERNAMELEN+1];
-	int		total_addresses;
-
-	/*
-	 * Compute ESMTP options.
-	 */
-	options[0] = '\0';
-	if (ctl->server.esmtp_options & ESMTP_8BITMIME) {
-             if (ctl->pass8bits || (ctl->mimemsg & MSG_IS_8BIT))
-		strcpy(options, " BODY=8BITMIME");
-             else if (ctl->mimemsg & MSG_IS_7BIT)
-		strcpy(options, " BODY=7BIT");
-        }
-
-	if ((ctl->server.esmtp_options & ESMTP_SIZE) && msg->reallen > 0)
-	    sprintf(options + strlen(options), " SIZE=%d", msg->reallen);
-
-	/*
-	 * Try to get the SMTP listener to take the Return-Path
-	 * address as MAIL FROM.  If it won't, fall back on the
-	 * remotename and mailserver host.  This won't affect replies,
-	 * which use the header From address anyway; the MAIL FROM
-	 * address is a place for the SMTP listener to send
-	 * bouncemail.  The point is to guarantee a FQDN in the MAIL
-	 * FROM line -- some SMTP listeners, like smail, become
-	 * unhappy otherwise.
-	 *
-	 * RFC 1123 requires that the domain name part of the
-	 * MAIL FROM address be "canonicalized", that is a
-	 * FQDN or MX but not a CNAME.  We'll assume the Return-Path
-	 * header is already in this form here (it certainly
-	 * is if rewrite is on).  RFC 1123 is silent on whether
-	 * a nonexistent hostname part is considered canonical.
-	 *
-	 * This is a potential problem if the MTAs further upstream
-	 * didn't pass canonicalized From/Return-Path lines, *and* the
-	 * local SMTP listener insists on them. 
-         *
-         * Handle the case where an upstream MTA is setting a return
-         * path equal to "@".  Ghod knows why anyone does this, but 
-	 * it's been reported to happen in mail from Amazon.com and
-	 * Motorola.
-	 */
-	if (!msg->return_path[0] || (0 == strcmp(msg->return_path, "@")))
-	{
-#ifdef HAVE_SNPRINTF
-	    snprintf(addr, sizeof(addr),
-#else
-	    sprintf(addr,
-#endif /* HAVE_SNPRINTF */
-		  "%s@%s", ctl->remotename, ctl->server.truename);
-	    ap = addr;
-	}
-	else if (strchr(msg->return_path,'@') || strchr(msg->return_path,'!'))
-	    ap = msg->return_path;
-	else		/* in case Return-Path existed but was local */
-	{
-#ifdef HAVE_SNPRINTF
-	    snprintf(addr, sizeof(addr),
-#else
-	    sprintf(addr,
-#endif /* HAVE_SNPRINTF */
-		    "%s@%s", msg->return_path, ctl->server.truename);
-	    ap = addr;
-	}
-
-	if (SMTP_from(ctl->smtp_socket, ap, options) != SM_OK)
-	{
-	    int err = handle_smtp_report(ctl, msg);
-
-	    SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
-	    return(err);
-	}
-
-	/*
-	 * Now list the recipient addressees
-	 */
-	total_addresses = 0;
-	for (idp = msg->recipients; idp; idp = idp->next)
-	    total_addresses++;
-	for (idp = msg->recipients; idp; idp = idp->next)
-	    if (idp->val.status.mark == XMIT_ACCEPT)
-	    {
-		if (strchr(idp->id, '@'))
-		    strcpy(addr, idp->id);
-		else {
-		    if (ctl->smtpname) {
-#ifdef HAVE_SNPRINTF
-		        snprintf(addr, sizeof(addr)-1, "%s", ctl->smtpname);
-#else
-			sprintf(addr, "%s", ctl->smtpname);
-#endif /* HAVE_SNPRINTF */
-
-		    } else {
-#ifdef HAVE_SNPRINTF
-		      snprintf(addr, sizeof(addr)-1, "%s@%s", idp->id, ctl->destaddr);
-#else
-		      sprintf(addr, "%s@%s", idp->id, ctl->destaddr);
-#endif /* HAVE_SNPRINTF */
-		    }
-		}
-		if (SMTP_rcpt(ctl->smtp_socket, addr) == SM_OK)
-		    (*good_addresses)++;
-		else
-		{
-		    char	errbuf[POPBUFSIZE];
-
-		    handle_smtp_report(ctl, msg);
-
-		    (*bad_addresses)++;
-		    idp->val.status.mark = XMIT_RCPTBAD;
-		    if (outlevel >= O_VERBOSE)
-			report(stderr, 
-			      _("%cMTP listener doesn't like recipient address `%s'\n"),
-			      ctl->listener, addr);
-		}
-	    }
-
-	/*
-	 * It's tempting to do local notification only if bouncemail was
-	 * insufficient -- that is, to add && total_addresses > *bad_addresses
-	 * to the test here.  The problem with this theory is that it would
-	 * make initial diagnosis of a broken multidrop configuration very
-	 * hard -- most single-recipient messages would just invisibly bounce.
-	 */
-	if (!(*good_addresses)) 
-	{
-	    if (!run.postmaster[0])
-	    {
-		if (outlevel >= O_VERBOSE)
-		    report(stderr, _("no address matches; no postmaster set.\n"));
-		SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
-		return(PS_REFUSED);
-	    }
-	    if (strchr(run.postmaster, '@'))
-		strncpy(addr, run.postmaster, sizeof(addr));
-	    else
-	    {
-#ifdef HAVE_SNPRINTF
-		snprintf(addr, sizeof(addr)-1, "%s@%s", run.postmaster, ctl->destaddr);
-#else
-		sprintf(addr, "%s@%s", run.postmaster, ctl->destaddr);
-#endif /* HAVE_SNPRINTF */
-	    }
-
-	    if (SMTP_rcpt(ctl->smtp_socket, addr) != SM_OK)
-	    {
-		report(stderr, _("can't even send to %s!\n"), run.postmaster);
-		SMTP_rset(ctl->smtp_socket);	/* required by RFC1870 */
-		return(PS_REFUSED);
-	    }
-
-	    if (outlevel >= O_VERBOSE)
-		report(stderr, _("no address matches; forwarding to %s.\n"), run.postmaster);
-	}
-
-	/* 
-	 * Tell the listener we're ready to send data.
-	 * Some listeners (like zmailer) may return antispam errors here.
-	 */
-	if (SMTP_data(ctl->smtp_socket) != SM_OK)
-	{
-	    SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
-	    return(handle_smtp_report(ctl, msg));
-	}
-    }
+	return(open_smtp_sink(ctl, msg, good_addresses, bad_addresses));
 
     /*
      * Awkward case.  User didn't specify an MDA.  Our attempt to get a
@@ -759,7 +997,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
      */
     else if (!ctl->mda)
     {
-	report(stderr, _("%cMTP connect to %s failed\n"),
+	report(stderr, GT_("%cMTP connect to %s failed\n"),
 	       ctl->listener,
 	       ctl->smtphost ? ctl->smtphost : "localhost");
 
@@ -780,175 +1018,13 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	 */
 	ctl->mda = FALLBACK_MDA;
 
-	report(stderr, _("can't raise the listener; falling back to %s"),
+	report(stderr, GT_("can't raise the listener; falling back to %s"),
 			 FALLBACK_MDA);
 #endif
     }
 
     if (ctl->mda)		/* must deliver through an MDA */
-    {
-	int	length = 0, fromlen = 0, nameslen = 0;
-	char	*names = NULL, *before, *after, *from = NULL;
-
-	ctl->destaddr = "localhost";
-
-	for (idp = msg->recipients; idp; idp = idp->next)
-	    if (idp->val.status.mark == XMIT_ACCEPT)
-		(*good_addresses)++;
-
-	length = strlen(ctl->mda);
-	before = xstrdup(ctl->mda);
-
-	/* get user addresses for %T (or %s for backward compatibility) */
-	if (strstr(before, "%s") || strstr(before, "%T"))
-	{
-	    /*
-	     * We go through this in order to be able to handle very
-	     * long lists of users and (re)implement %s.
-	     */
-	    nameslen = 0;
-	    for (idp = msg->recipients; idp; idp = idp->next)
-		if ((idp->val.status.mark == XMIT_ACCEPT))
-		    nameslen += (strlen(idp->id) + 1);	/* string + ' ' */
-	    if ((*good_addresses == 0))
-		nameslen = strlen(run.postmaster);
-
-	    names = (char *)xmalloc(nameslen + 1);	/* account for '\0' */
-	    if (*good_addresses == 0)
-		strcpy(names, run.postmaster);
-	    else
-	    {
-		names[0] = '\0';
-		for (idp = msg->recipients; idp; idp = idp->next)
-		    if (idp->val.status.mark == XMIT_ACCEPT)
-		    {
-			strcat(names, idp->id);
-			strcat(names, " ");
-		    }
-		names[--nameslen] = '\0';	/* chop trailing space */
-	    }
-
-	    /* sanitize names in order to contain only harmless shell chars */
-	    sanitize(names);
-	}
-
-	/* get From address for %F */
-	if (strstr(before, "%F"))
-	{
-	    from = xstrdup(msg->return_path);
-
-	    /* sanitize from in order to contain *only* harmless shell chars */
-	    sanitize(from);
-
-	    fromlen = strlen(from);
-	}
-
-	/* do we have to build an mda string? */
-	if (names || from) 
-	{		
-	    char	*sp, *dp;
-
-	    /* find length of resulting mda string */
-	    sp = before;
-	    while ((sp = strstr(sp, "%s"))) {
-		length += nameslen - 2;	/* subtract %s */
-		sp += 2;
-	    }
-	    sp = before;
-	    while ((sp = strstr(sp, "%T"))) {
-		length += nameslen - 2;	/* subtract %T */
-		sp += 2;
-	    }
-	    sp = before;
-	    while ((sp = strstr(sp, "%F"))) {
-		length += fromlen - 2;	/* subtract %F */
-		sp += 2;
-	    }
-		
-	    after = xmalloc(length + 1);
-
-	    /* copy mda source string to after, while expanding %[sTF] */
-	    for (dp = after, sp = before; (*dp = *sp); dp++, sp++) {
-		if (sp[0] != '%')	continue;
-
-		/* need to expand? BTW, no here overflow, because in
-		** the worst case (end of string) sp[1] == '\0' */
-		if (sp[1] == 's' || sp[1] == 'T') {
-		    strcpy(dp, names);
-		    dp += nameslen;
-		    sp++;	/* position sp over [sT] */
-		    dp--;	/* adjust dp */
-		} else if (sp[1] == 'F') {
-		    strcpy(dp, from);
-		    dp += fromlen;
-		    sp++;	/* position sp over F */
-		    dp--;	/* adjust dp */
-		}
-	    }
-
-	    if (names) {
-		free(names);
-		names = NULL;
-	    }
-	    if (from) {
-		free(from);
-		from = NULL;
-	    }
-
-	    free(before);
-
-	    before = after;
-	}
-
-
-	if (outlevel >= O_DEBUG)
-	    report(stdout, _("about to deliver with: %s\n"), before);
-
-#ifdef HAVE_SETEUID
-	/*
-	 * Arrange to run with user's permissions if we're root.
-	 * This will initialize the ownership of any files the
-	 * MDA creates properly.  (The seteuid call is available
-	 * under all BSDs and Linux)
-	 */
-	seteuid(ctl->uid);
-#endif /* HAVE_SETEUID */
-
-	sinkfp = popen(before, "w");
-	free(before);
-	before = NULL;
-
-#ifdef HAVE_SETEUID
-	/* this will fail quietly if we didn't start as root */
-	seteuid(0);
-#endif /* HAVE_SETEUID */
-
-	if (!sinkfp)
-	{
-	    report(stderr, _("MDA open failed\n"));
-	    return(PS_IOERR);
-	}
-
-	/*
-	 * We need to disable the normal SIGCHLD handling here because 
-	 * sigchld_handler() would reap away the error status, returning
-	 * error status instead of 0 for successful completion.
-	 */
-#ifndef HAVE_SIGACTION
-	sigchld = signal(SIGCHLD, SIG_DFL);
-#else
-	memset (&sa_new, 0, sizeof sa_new);
-	sigemptyset (&sa_new.sa_mask);
-	sa_new.sa_handler = SIG_DFL;
-	sigaction (SIGCHLD, &sa_new, NULL);
-#endif /* HAVE_SIGACTION */
-    }
-
-    /*
-     * We need to stash this away in order to know how many
-     * response lines to expect after the LMTP end-of-message.
-     */
-    lmtp_responses = *good_addresses;
+	return(open_mda_sink(ctl, msg, good_addresses, bad_addresses));
 
     return(PS_SUCCESS);
 }
@@ -990,7 +1066,7 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	if (rc)
 	{
 	    report(stderr, 
-		   _("MDA returned nonzero status %d\n"), rc);
+		   GT_("MDA returned nonzero status %d\n"), rc);
 	    return(FALSE);
 	}
     }
@@ -1006,7 +1082,7 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	if (error)
 	{
 	    report(stderr, 
-		   _("Message termination or close of BSMTP file failed\n"));
+		   GT_("Message termination or close of BSMTP file failed\n"));
 	    return(FALSE);
 	}
     }
@@ -1022,7 +1098,7 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	    }
 	    else
 	    {
-		report(stderr, _("SMTP listener refused delivery\n"));
+		report(stderr, GT_("SMTP listener refused delivery\n"));
 	        SMTP_rset(ctl->smtp_socket);    /* stay on the safe side */
 		return(TRUE);
 	    }
@@ -1052,10 +1128,10 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 		 * comply.
 		 */
 		if (atoi(smtp_response) == 503)
-		    report(stderr, _("LMTP delivery error on EOM\n"));
+		    report(stderr, GT_("LMTP delivery error on EOM\n"));
 		else
 		    report(stderr,
-			  _("Unexpected non-503 response to LMTP EOM: %s\n"),
+			  GT_("Unexpected non-503 response to LMTP EOM: %s\n"),
 			  smtp_response);
 
 		/*
@@ -1134,20 +1210,20 @@ int open_warning_by_mail(struct query *ctl, struct msgblk *msg)
      * it's worth to compute.
      */
     struct msgblk reply = {NULL, NULL, "FETCHMAIL-DAEMON@", 0};
+    int status;
 
     strcat(reply.return_path, fetchmailhost);
 
     if (!MULTIDROP(ctl))		/* send to calling user */
     {
-	int status;
-
 	save_str(&reply.recipients, ctl->localnames->id, XMIT_ACCEPT);
 	status = open_sink(ctl, &reply, &good, &bad);
 	free_str_list(&reply.recipients);
-	return(status);
     }
     else				/* send to postmaster  */
-	return(open_sink(ctl, &reply, &good, &bad));
+	status = open_sink(ctl, &reply, &good, &bad);
+    stuff_warning(ctl, "Date: %s", rfc822timestamp());
+    return(status);
 }
 
 #if defined(HAVE_STDARG_H)
@@ -1193,7 +1269,7 @@ va_dcl
 void close_warning_by_mail(struct query *ctl, struct msgblk *msg)
 /* sign and send mailed warnings */
 {
-    stuff_warning(ctl, _("--\r\n\t\t\t\tThe Fetchmail Daemon\r\n"));
+    stuff_warning(ctl, GT_("--\r\n\t\t\t\tThe Fetchmail Daemon\r\n"));
     close_sink(ctl, msg, TRUE);
 }
 
